/** A simple animal that moves randomly and eats.
 */

import swarm.Globals;
import swarm.gui.Raster;

import swarm.space.Grid2d;
import swarm.space.Grid2dImpl;


public class Animal
{
    public int currX, currY;       // the current positon of the animal in the landscape
    public byte animalColor;       // the color of the animal

    Grid2d world;                  // the world where the animal lives
    Cell currCell, neighborCell;   // the cell it is currently on and its neighbors   

    // the constructor for the animal
    public Animal(Grid2d w)
    {
	world = w;

    }

    public void setXY(int myX, int myY)
    {
	currX = myX;
	currY = myY;
	
	currCell = (Cell) world.getObjectAtX$Y(currX, currY);

    }

    public void setColor(byte c) 
    { 
	animalColor = c; 

    }

    public int getX()
    {
	return currX;

    }


    public int getY()
    {
	return currY;

    }


    public byte getColor()
    {
	return animalColor;
    }

    private boolean withinBounds(int x, int y)
    {
	boolean iamInside;

	if ((x >= 0) && (y >= 0) &&
	    (x < world.getSizeX()) && (y < world.getSizeY()))
	    {
		iamInside = true;

	    }
	else
	    {
		iamInside = false;
	    }

	return iamInside;
    }

    public void step()
    {
	int rndX, rndY;

	// move randomly
	rndX = currX + Globals.env.uniformIntRand.getIntegerWithMin$withMax(-1, 1);
	rndY = currY + Globals.env.uniformIntRand.getIntegerWithMin$withMax(-1, 1);

	if (withinBounds(rndX, rndY))
	    {
		neighborCell = (Cell ) world.getObjectAtX$Y(rndX, rndY);
		if (neighborCell.getAnimal() == null)
		    {
			currCell.setAnimal(null);

			this.setXY(rndX, rndY);
			currCell.setAnimal(this);
			
		    }
	    }
	this.eat();
    }

    private void eat()
    {
	double rndEat;

	rndEat = Globals.env.uniformDblRand.getDoubleWithMin$withMax(0.01, 0.1);
	// nothing happens to the animal here
	// only the amount of vegetation in the cell reduces
	currCell.vegReduce(rndEat);

    }

    public Object drawSelfOn (Raster r) 
    {
	r.drawPointX$Y$Color (currX, currY, animalColor);
	return this;
    }

}





