/** A Cell is a unit block of land. 
    A landscape may be considered as a rectangular arrangement of cells
 */

import swarm.objectbase.SwarmObjectImpl;
import swarm.objectbase.SwarmObject;

import swarm.gui.Raster;

import swarm.defobj.Zone;

public class Cell extends SwarmObjectImpl
{
    public int x, y;           // my co-ords
    public byte myColor, maxColor;      

    public double amountOfVeg; // amount of vegetation in this cell 

    Landscape myLand;          // the land of which this cell is a part
    public Animal myAnimal;    // the animal that drops by here


    // constructor for the class
    public Cell(Zone aZone)
    {
	super (aZone);
	
    }

    public void setLand(Landscape l)
    {
	myLand = l;
    }

    public void setXY(int myX, int myY)
    {
	x = myX;
	y = myY;

    }

    public void setInitialVegetation(double iniAmount)
    {
	amountOfVeg = iniAmount;
    }
    
    public int getX()
    {
	return x;
    }

    public int getY()
    {
	return y;
    }

    public void setColor(byte c)
    {
	maxColor = c;
	myColor = c;

    }

    public byte getColor()
    {
	return myColor;
    }

    public void setAnimal(Object a)
    {
	myAnimal = (Animal) a;
    }

    public Animal getAnimal()
    {
	return myAnimal;
    }

    public void step()
    {
	// allow the vegetation to grow a bit
	// growth could be based on soil type, precipitation etc, but here its hard-coded
	vegGrow(0.01);
	
    }

    private void vegGrow(double growth)
    {
	amountOfVeg = amountOfVeg + growth;
    }

    public void vegReduce(double amount)
    {
	amountOfVeg = amountOfVeg - amount;
	amountOfVeg = (amountOfVeg < 0.0) ? 0.0 : amountOfVeg;

    }

    public Object drawSelfOn (Raster r) 
    {
	myColor = (byte) (amountOfVeg * maxColor);
	myColor = (myColor > maxColor) ? maxColor : myColor;

	if (this.getAnimal() == null)
	    {
		r.drawPointX$Y$Color(x,y, (byte) myColor);
	    }
	else
	    {
		r.drawPointX$Y$Color(x,y, (byte) this.getAnimal().getColor());
	    }
	return this;
    }


}


