import swarm.Globals;          

import swarm.Selector;

import swarm.defobj.Zone;


import swarm.defobj.FArguments; 
import swarm.defobj.FArgumentsImpl;
import swarm.defobj.FCall;
import swarm.defobj.FCallImpl;

import swarm.activity.Activity;
import swarm.activity.ActionGroup;
import swarm.activity.ActionGroupImpl;
import swarm.activity.Schedule;
import swarm.activity.ScheduleImpl;
import swarm.activity.ActionForEach; 
import swarm.activity.ActionTo;     

import swarm.objectbase.Swarm; 
import swarm.objectbase.SwarmImpl;
import swarm.objectbase.ProbeMap; 
import swarm.objectbase.ProbeMapImpl; 
import swarm.objectbase.EmptyProbeMapImpl;
import swarm.objectbase.VarProbe;

import swarm.space.Grid2d; 
import swarm.space.Grid2dImpl;

import java.util.LinkedList; 
import java.util.List;


// import java.lang.*;

public class ModelSwarm extends SwarmImpl
{
    public int worldX, worldY;
    public int numAnimals;

    List animalList;
    Animal anAnimal;
    Cell aCell;
    Landscape theLand;

    ProbeMap probeModel;

    public ActionGroup modelActions;
    public Schedule modelSchedule;
    ActionForEach actionForEach;
    ActionTo actionTo;

    // constructor for the class
    public ModelSwarm(Zone aZone)
    {
	super (aZone);

	numAnimals = 10;
	worldX = 100;
	worldY = 100;

	class ModelProbeMap extends EmptyProbeMapImpl
	{
	      private VarProbe probeVariable (String name)
	      {
		  return
		      Globals.env.probeLibrary.getProbeForVariable$inClass
		      (name, ModelSwarm.this.getClass());
	      }

	      private void addVar (String name)
	      {
		  addProbe (probeVariable (name));
		  
	      }

	      public ModelProbeMap (Zone pZone, Class aClass)
	      {
		  super (pZone, aClass);
		  addVar ("worldX");
		  addVar ("worldY");
		  addVar ("numAnimals");
	      }
	}

	probeModel = (ProbeMap) new ModelProbeMap (aZone, getClass());
	Globals.env.probeLibrary.setProbeMap$For (probeModel, getClass());

    }

    public int getnumAnimals()
    {
	return numAnimals;
    }

    public List getAnimalList()
    {
	return animalList;

    }

    public Landscape getWorld()
    {
	return theLand;

    }

    public void updateWorld()
    {
	for (int i = 0; i < worldX; i++)
	    {
		for (int j = 0; j < worldY; j++)
		    {
			aCell = (Cell) theLand.getObjectAtX$Y(i, j);
			aCell.step();
		    }
	    }
    }

    public Object buildObjects()
    {
	int rndX, rndY;

	super.buildObjects();

	theLand = new Landscape(this.getZone(), worldX, worldY);
	theLand.createCells();

	// add the animals
	animalList = new LinkedList();
	do {
	    rndX = Globals.env.uniformIntRand.getIntegerWithMin$withMax (0, (theLand.getSizeX()-1));
	    rndY = Globals.env.uniformIntRand.getIntegerWithMin$withMax (0, (theLand.getSizeY()-1));

	    aCell = (Cell) theLand.getObjectAtX$Y(rndX, rndY);
	    if (aCell.getAnimal() == null)
		{
		    anAnimal = new Animal(theLand);
		    anAnimal.setXY(rndX, rndY);

		    animalList.add(anAnimal);
		    aCell.setAnimal(anAnimal);
		}
	} while (animalList.size() < numAnimals);

	return this;

    }

    public Object buildActions()
    {
	super.buildActions();

	modelActions = new ActionGroupImpl(this.getZone());

	try
	    {
		Selector animalSel = new Selector(anAnimal.getClass(), "step", false);
		actionForEach = modelActions.createActionForEach$message(animalList, animalSel);
	    }
	catch (Exception e)
	    {
		System.err.println("Exception at actionForEach: " + e.getMessage());
	    }

	try
	    {
		actionTo = modelActions.createActionTo$message
		    (this, new Selector(this.getClass(), "updateWorld", false));

	    }
	catch(Exception e)
	    {
		System.err.println("Exception occured at actionTo: " + e.getMessage());
	    }

	modelSchedule = new ScheduleImpl (this.getZone(), 1);
	modelSchedule.at$createAction (0, modelActions);
	return this;

    }

    public Activity activateIn(Swarm swarmContext)
    {
	super.activateIn(swarmContext);

	modelSchedule.activateIn(this);
	return this.getActivity();
    }

}





