import swarm.Globals;

import swarm.Selector;
import swarm.defobj.Zone;

import swarm.activity.Activity;
import swarm.activity.ActionGroup;
import swarm.activity.ActionGroupImpl;
import swarm.activity.Schedule;
import swarm.activity.ScheduleImpl;

import swarm.gui.Colormap;
import swarm.gui.ColormapImpl;
import swarm.gui.ZoomRaster;
import swarm.gui.ZoomRasterImpl;

import swarm.objectbase.Swarm;
import swarm.objectbase.ProbeMap; 
import swarm.objectbase.ProbeMapImpl;
import swarm.objectbase.EmptyProbeMapImpl;
import swarm.objectbase.VarProbe;

import swarm.simtoolsgui.GUISwarm;
import swarm.simtoolsgui.GUISwarmImpl;

// import swarm.space.Value2dDisplay;
// import swarm.space.Value2dDisplayImpl;
import swarm.space.Object2dDisplay;
import swarm.space.Object2dDisplayImpl;

// import swarm.analysis.EZGraph;
// import swarm.analysis.EZGraphImpl;

import java.util.List;


public class ObserverSwarm extends GUISwarmImpl
{

    public int displayFrequency;	
    public int timeStep;
    public ProbeMap probeObsvr;

    public ActionGroup displayActions;				
    public Schedule displaySchedule;

    public ModelSwarm modelSwarm;	  	

    public Colormap colormap;		
    public ZoomRaster worldRaster;

    public Object2dDisplay cellDisplay;


    /* Constructor */
    public ObserverSwarm(Zone aZone)
    {
	super (aZone);
	displayFrequency = 1;
	timeStep = 0;

	class ObserverProbeMap extends EmptyProbeMapImpl
	{
	    private VarProbe probeVariable(String name)
	    {
		return Globals.env.probeLibrary.getProbeForVariable$inClass
		    (name, ObserverSwarm.this.getClass());
	    }

	    private void addVar (String name)
	    {
		addProbe (probeVariable(name));
	    }

	    public ObserverProbeMap(Zone pZone, Class aClass)
	    {
		super (pZone, aClass);
		addVar ("displayFrequency");
		addVar ("timeStep");
	    }

	}
	  
	probeObsvr = (ProbeMap) new ObserverProbeMap(aZone, getClass());
	Globals.env.probeLibrary.setProbeMap$For(probeObsvr, getClass());


    }

    public Object _worldRasterDeath_ (Object caller) 
    {
	worldRaster.drop ();
	worldRaster = null;
	return this;
    }


    public Object buildObjects()
    {
	super.buildObjects();

	modelSwarm = new ModelSwarm(this.getZone());
	  
	Globals.env.createArchivedProbeDisplay(modelSwarm, "ModelSwarm");
	Globals.env.createArchivedProbeDisplay(this, "ObserverSwarm");
	  
	this.getControlPanel ().setStateStopped ();

	modelSwarm.buildObjects();

	colormap = new ColormapImpl(this.getZone());

	// create 16 colors of green
	for (int i = 0; i < 16; i++)
	    colormap.setColor$ToRed$Green$Blue( (byte) i, (double) 0, i/15.0, 0 );

	// set the color for the cells
	Landscape land = modelSwarm.getWorld();
	for (int i = 0; i < land.getSizeX(); i++)
	    {
		for (int j = 0; j < land.getSizeY(); j++)
		    {
			Cell aCell = (Cell) land.getObjectAtX$Y(i, j);
			aCell.setColor((byte) 15);
		    }
	    }

	// set a color for the ant
	colormap.setColor$ToName((byte) 16, "white");      // for the animals
	List animalList = modelSwarm.getAnimalList();
	for(int i = 0;i < animalList.size(); i++)
	    {
		Animal anAnimal = (Animal) animalList.get(i);
		anAnimal.setColor((byte) 16);

	    }

	worldRaster = new ZoomRasterImpl (getZone (), "worldRaster");
	try 
	    {
		worldRaster.enableDestroyNotification$notificationMethod 
		    (this,
		     new Selector (getClass (), "_worldRasterDeath_", false));
	    } 
	catch (Exception e) 
	    {
		System.err.println ("Exception _worldRasterDeath_: " 
				    + e.getMessage ());
	    }
        
	worldRaster.setColormap (colormap);
	worldRaster.setZoomFactor (3);
	worldRaster.setWidth$Height 
	    ((modelSwarm.getWorld ()).getSizeX (),
	     (modelSwarm.getWorld ()).getSizeY ());
	worldRaster.setWindowTitle ("Landscape");
	worldRaster.pack();			
       

	try
	    {
		cellDisplay = new Object2dDisplayImpl
		    (this.getZone(), worldRaster, modelSwarm.getWorld(),
		     new Selector (Class.forName("Cell"), "drawSelfOn", false));
	    }
	catch (Exception e)
	    {
		System.out.println("Exception drawSelfOn for Cells: " + e.getMessage());
	    }

	try 
	    {
		worldRaster.setButton$Client$Message 
		    (3, cellDisplay, new Selector (cellDisplay.getClass (), 
						   "makeProbeAtX$Y", true));
	    } 
	catch (Exception e) 
	    {
		System.err.println ("Exception makeProbeAtX$Y: " 
				    + e.getMessage ());
	    }

	return this;

    }

    public Object _update_ ()  
    {
	timeStep = timeStep + displayFrequency;

	if (worldRaster != null) 
	    {
		cellDisplay.display();
		worldRaster.drawSelf ();
	    }

	return this;

    }

    public Object buildActions () 
    {
	super.buildActions();
    
	modelSwarm.buildActions();
  
	displayActions = new ActionGroupImpl (this.getZone());
	try 
	    {
		// refresh the display
		displayActions.createActionTo$message 
		    (this, new Selector (getClass (), "_update_", false));
		  
		// update the probes
		displayActions.createActionTo$message
		    (Globals.env.probeDisplayManager, 
		     new Selector (Globals.env.probeDisplayManager.getClass (),
				   "update", true));
		  
		displayActions.createActionTo$message
		    (getActionCache (), new Selector 
			(getActionCache ().getClass (), "doTkEvents", true));
	    } 
	catch (Exception e) 
	    {
		System.err.println ("Exception in setting up displayActions : " 
				    + e.getMessage ());
	    }


	displaySchedule = new ScheduleImpl (getZone (), displayFrequency);
	displaySchedule.at$createAction (0, displayActions);
  
	return this;

    }  

    public Activity activateIn (Swarm swarmContext) 
    {
	super.activateIn (swarmContext);

	modelSwarm.activateIn (this);

	displaySchedule.activateIn (this);
	return getActivity();

    }
    

    public void drop () 
    {
	if (worldRaster != null)
	    worldRaster.disableDestroyNotification ();

	super.drop ();

    }


}



